#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
try:
    from urllib import request as _urllib  # noqa
except ImportError:
    import urllib as _urllib
import os
import subprocess
import time

from charmhelpers.contrib.openstack import utils as helpers_utils
from charmhelpers.core import hookenv
from charmhelpers.core import host
from charmhelpers.core.services import base
from charmhelpers.core.services import helpers
from midonet_helpers import puppet
from midonet_helpers import relations
from midonet_helpers import repositories
import utils


CERT_PATH = '/etc/pki/mem-manager/apache.crt'
KEY_PATH = '/etc/pki/mem-manager/apache.key'
PKI_PATH = '/etc/pki/mem-manager'


class MEManagerConfig(dict):
    def __init__(self, *args):
        self._cert_key_path = None
        self._cert_path = None
        self['config'] = hookenv.config()

        server_name = self['config'].get('server-name')
        if not server_name:
            server_name = utils.get_ip()
        self['config']['name'] = server_name

        self._get_certs()

    def _get_certs(self):
        if (not self['config'].get('ssl-certificate-url') or
                not self['config'].get('ssl-certificate-key-url')):
            # Generate self signed certificates
            if not os.path.exists(CERT_PATH) or not os.path.exists(KEY_PATH):
                cmd = ['openssl', 'req', '-x509', '-nodes', '-days', '365',
                       '-newkey', 'rsa:2048', '-keyout', KEY_PATH,
                       '-out', CERT_PATH, '-subj',
                       '/C=CZ/ST=kralovehradecky/L=Trutnov/'
                       'O=%s/OU=Networking/CN=%s' %
                       (self['config']['mem-username'],
                        self['config']['name'])]
                try:
                    subprocess.check_call(cmd)
                except subprocess.CalledProcessError as e:
                    hookenv.log('Failed to generate a mock ssl certificate. '
                                'Caught error: %s' % e)
                    self['config'] = {}  # Clear config so nothing is done
                    hookenv.status_set(
                        'maintenance',
                        'No ssl cert and failed to generate one')
        else:
            try:
                _urllib.urlretrieve(self['config']['ssl-certificate-url'],
                                    CERT_PATH)
                _urllib.urlretrieve(self['config']['ssl-certificate-key-url'],
                                    KEY_PATH)
            except Exception as e:
                hookenv.log('Failed to retrieve the ssl certificate. %s' % e)
                self['config'] = {}  # Clear config so nothing is done
                return

        host.chownr(PKI_PATH, 'www-data', 'www-data')
        self['config']['cert_path'] = CERT_PATH
        self['config']['key_path'] = KEY_PATH


def manage():
    mem_config = MEManagerConfig()
    ost_origin = mem_config['config'].get('openstack-origin')
    midonet_origin = mem_config['config'].get('midonet-origin')
    mem_username = mem_config['config'].get('mem-username')
    mem_pwd = mem_config['config'].get('mem-password')
    if None in (ost_origin, midonet_origin) or (
            midonet_origin is not None and midonet_origin[:3] == 'mem' and None
            in (mem_username, mem_pwd)):
        hookenv.status_set('maintenance', 'lacking repo configuration')
        return
    if not midonet_origin.startswith('mem'):
        hookenv.status_set('maintenance', 'Invalid mem release specified in '
                           'configuration')
        return
    ost_release = helpers_utils.get_os_codename_install_source(ost_origin)
    repo = repositories.config('%s/%s' % (ost_release, midonet_origin),
                               mem_username, mem_pwd)
    api = relations.MidonetApiRelation()

    manager = base.ServiceManager([
        {
            'service': 'apache2',
            'ports': [443],
            'provided_data': [
                helpers.HttpRelation()],
            'required_data': [repo,
                              mem_config,
                              api],
            'data_ready': [
                puppet.InstallPackagesCallback(packages=('midonet-manager',)),
                helpers.render_template(
                    source='000-default.conf',
                    target='/etc/apache2/sites-enabled/000-default.conf'),
                helpers.render_template(
                    source='repo.yaml',
                    target='/var/lib/hiera/common.yaml'),
                helpers.render_template(
                    source='client.js',
                    target='/var/www/html/midonet-manager/config/client.js'),
            ],
            'start': [base.service_restart,
                      base.PortManagerCallback()],
        },
    ])
    manager.manage()

    if host.service_running('apache2') and api:
        # Try if the proxying works
        attempts = 5
        while attempts:
            try:
                _urllib.urlopen('https://127.0.0.1/midonet-api/').read()
            except IOError as ioe:
                if 401 in ioe.args:
                    # We got "unauthorized, it means the endpoint is listening
                    hookenv.status_set(
                        'active',
                        'Apache2 is up and running the MidoNet manager')
                    time.sleep(5)
                elif ioe.args[0].errno == os.errno.ECONNREFUSED:
                    time.sleep(5)
                    host.service_restart('apache2')
                else:
                    hookenv.log(
                        'Failed to connect to MidoNet API through the reverse '
                        'proxy. Got error: %s' % ioe)
                    hookenv.status_set(
                        'maintenance',
                        'No access to MidoNet API available yet.')
                    break
            except Exception as exc:
                hookenv.log('Failed to connect to MidoNet API through the '
                            'reverse proxy. Got error: %s' % exc)
                hookenv.status_set('maintenance',
                                   'No access to MidoNet API available yet.')
                break
            attempts -= 1
    else:
        hookenv.log('Apache not running or API relation incomplete.')
        hookenv.status_set('maintenance',
                           'No access to MidoNet API available yet.')
