#!/usr/bin/env python3
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import os
import uuid

import amulet

from charmhelpers.contrib.openstack.amulet.utils import (
    OpenStackAmuletUtils,
    DEBUG,  # flake8: noqa
    ERROR
)

# Use DEBUG to turn on debug logging
u = OpenStackAmuletUtils(DEBUG)

SETUP_TIMEOUT = 9600
USERNAME = 'admin'
PASSWORD = 'openstack'
PROJECT_NAME = 'admin'


class ManagerDeployment():
    def __init__(self, ubuntu_series, openstack_origin=None,
                 midonet_origin=None):
        self.os_release = openstack_origin
        self.series = ubuntu_series
        self.midonet_release = (openstack_origin.split('-')[-1] + '/' +
            midonet_origin)
        self.midonet_origin = midonet_origin
        self.mem = self.midonet_origin[:3] == 'mem'
        self.mem_credentials = {'mem-username': os.getenv('MEM_USERNAME'),
                                'mem-password': os.getenv('MEM_PWD')}
        self.d = amulet.Deployment(series=self.series)
        self.d.charm_name = 'mem-manager'

        # Sentries
        self.zookeeper_sentry = None
        self.keystone_sentry = None
        self.mn_api_sentry = None
        self.keystone_api_relation = None
        self.zookeeper_api_relation = None

        self._add_services()
        self._add_relations()
        self._configure_services()
        self._deploy()
        self._initialize_tests()

    def _add_services(self):
        self.d.add('mysql', charm='mysql', units=1,
                   series=self.series)
        self.d.add('keystone', charm='keystone', units=1,
                   series=self.series)
        self.d.add('zookeeper', charm='zookeeper', units=1,
                   series=self.series)
        self.d.add('midonet-api', charm='midonet-api',
                   series=self.series)
        self.d.add('mem-manager')

    def _add_relations(self):
        self.d.relate('mysql:shared-db', 'keystone:shared-db')
        self.d.relate('keystone:identity-service', 'midonet-api:keystone')
        self.d.relate('zookeeper:zookeeper', 'midonet-api:zookeeper')

        self.d.relate('midonet-api:midonet-api', 'mem-manager:midonet')


    def _configure_services(self):
        self.d.configure('keystone', {
                         'admin-user': USERNAME,
                         'admin-password': PASSWORD,
                         'admin-token': 'osttesting',
                         'enable-pki': 'false',
                         'openstack-origin': self.os_release})

        self.d.configure('mysql', {'max-connections': 500})

        api_config = {'midonet-origin': self.midonet_origin,
                      'openstack-origin': self.os_release}

        api_config.update(self.mem_credentials)
        self.d.configure('midonet-api', api_config)
        self.d.configure('mem-manager', api_config)


    def _deploy(self):
        try:
            self.d.setup(timeout=SETUP_TIMEOUT)
            self.d.sentry.wait(timeout=SETUP_TIMEOUT)
        except amulet.helpers.TimeoutError:
            amulet.raise_status(amulet.SKIP,
                                msg="Environment wasn't stood up in time")

    def _initialize_tests(self):
        self.zookeeper_sentry = self.d.sentry.unit['zookeeper/0']
        self.keystone_sentry = self.d.sentry.unit['keystone/0']
        self.mn_api_sentry = self.d.sentry.unit['midonet-api/0']
        self.manager_sentry = self.d.sentry.unit['mem-manager/0']

        self.keystone_api_relation = self.keystone_sentry.relation(
            'identity-service', 'midonet-api:keystone')
        self.zookeeper_api_relation = self.zookeeper_sentry.relation(
            'zookeeper', 'midonet-api:zookeeper')

    def run_tests(self):
        for test in dir(self):
            if test.startswith('test_'):
                getattr(self, test)()

    def test_manager(self):
        u.log.debug('Checking if the repo key has been correctly fetched...')
        apt_key_out, _ = self.manager_sentry.run(
            'apt-key list | grep Midokura')
        if 'Midokura' not in apt_key_out:
            message = ("error, Midokura repository key is not installed")
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking if mem-manager package has been installed...')
        dpkg_api_out, _ = self.manager_sentry.run(
            'dpkg -l | grep midonet-manager')
        if 'midonet-manager' not in dpkg_api_out:
            message = ("error: midonet-manager package is not installed")
            amulet.raise_status(amulet.FAIL, msg=message)

        u.log.debug('Checking proxying to the midonet-api')
        cli_out, mn_exit = self.manager_sentry.run(
            'curl -k https://127.0.0.1/midonet-api/login')
        if mn_exit != 0:
            message = ("midonet-cli error: %s" % cli_out)
            amulet.raise_status(amulet.FAIL, msg=message)
        if '"code":401' not in cli_out:
            amulet.raise_status(amulet.FAIL,
                                'Failed to proxy midonet-api')

        cmd = """curl -H "Content-Type: application/json" -d '
{ "auth": {
    "passwordCredentials": {
        "username": "%s",
        "password": "%s"
    },
    "tenantName": "%s"
}}' http://127.0.0.1:5000/v2.0/tokens 2> /dev/null""" % (
            USERNAME, PASSWORD, PROJECT_NAME)
        cli_out, rc = self.keystone_sentry.run(cmd)
        if rc != 0:
            message = ("Error fetching keystone token: %s" % cli_out)
            amulet.raise_status(amulet.FAIL, msg=message)
        try:
            data = json.loads(cli_out)
            token = data['access']['token']['id']
        except Exception as exc:
            message = ("Error parsing keystone token: %s" % exc)
            amulet.raise_status(amulet.FAIL, msg=message)

        cli_out, rc = self.manager_sentry.run(
            'curl -k -H "X-Auth-Token: %s" -H "X-Auth-Project: %s"'
            ' https://127.0.0.1/midonet-api/system_state' % (token,
                                                             PROJECT_NAME))
        if rc != 0:
            message = ("Error fetching keystone token: %s" % token)
            amulet.raise_status(amulet.FAIL, msg=message)
        data = json.loads(cli_out)
        if not data['uri'].startswith('/midonet-api'):
            message = ("Failed to turn absolute into relative urls: %s" % data)
            amulet.raise_status(amulet.FAIL, msg=message)
