#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4 filetype=python
#
# Copyright (c) 2015 Midokura Europe SARL, All Rights Reserved.
# All Rights Reserved
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import functools
import shutil
import tempfile

import mock
import services

from .midonet_helpers import test

HOOKENV_PATCHES = [
    'charm_dir',
    'config',
    'is_relation_made',
    'log',
    'open_port',
    'relation_get',
    'relation_ids',
    'related_units',
    'relation_set',
    'unit_get']


class HookTests(test.CharmTestCase):
    @mock.patch('charmhelpers.fetch.apt_install')
    def setUp(self, apt_install):
        self.relations = test.FakeRelation(self._full_relation_data())
        test.CharmTestCase.setUp(self, services.hookenv, HOOKENV_PATCHES)

        def lsb_release():
            return {
                'DISTRIB_ID': 'Ubuntu',
                'DISTRIB_RELEASE': '14.04',
                'DISTRIB_CODENAME': 'trusty',
                'DISTRIB_DESCRIPTION': "Ubuntu 14.04.3 LTS",
            }

        def get_os_codename_package(pkg, fatal=True):
            return 'liberty'

        self.patch_all(services.helpers_utils,
                       ('get_os_codename_package',
                        'lsb_release',))
        self.lsb_release.side_effect = lsb_release
        self.get_os_codename_package.side_effect = get_os_codename_package

        _templating = mock.patch.object(services.helpers, 'render_template')
        setattr(self, 'render_template', _templating.start())
        self.addCleanup(_templating.stop)

        _check_call = mock.patch.object(services.hookenv.subprocess,
                                        'check_call')
        setattr(self, 'check_call', _check_call.start())
        self.addCleanup(_check_call.stop)

        _puppet = mock.patch.object(services.puppet, '_apply')
        setattr(self, 'apply', _puppet.start())
        self.addCleanup(_puppet.stop)

        _relation_helpers = mock.patch.object(
            services.relations.helpers, 'hookenv', spec=[
                'DEBUG',
                'log',
                'relation_ids',
                'related_units',
                'relation_get'])
        setattr(self, 'relation_helpers', _relation_helpers.start())
        self.addCleanup(_relation_helpers.stop)

        self.test_config.config['openstack-origin'] = 'cloud:trusty-liberty'
        self.test_config.config['server-name'] = 'dom.example.com'
        self.test_config.config['mem-username'] = 'foo'
        self.test_config.config['mem-password'] = 'bar'
        self.config.side_effect = self.test_config.get
        self.relation_get.side_effect = self.test_relation.get

        self.charm_dir.return_value = tempfile.mkdtemp()
        self.related_units.side_effect = self.relations.related_units
        self.relation_ids.side_effect = self.relations.relation_ids
        self.relation_get.side_effect = self.relations.get

    def tearDown(self):
        shutil.rmtree(self.charm_dir.return_value)
        test.CharmTestCase.tearDown(self)

    @mock.patch('os.makedirs')
    @mock.patch('services.host.chownr')
    @mock.patch.object(services._urllib, 'urlretrieve')
    @mock.patch('charmhelpers.core.services.base.service_restart')
    @mock.patch('charmhelpers.core.services.base.service_stop')
    def test_config_change(self, service_stop, service_restart, urlretrieve,
                           chown, makedirs):
        urlretrieve.return_value = ('tmp/foo', '')

        self.relation_helpers.relation_ids.side_effect = (
            self.relations.relation_ids)
        self.relation_helpers.related_units.side_effect = (
            self.relations.related_units)
        self.relation_helpers.relation_get.side_effect = (
            self.relations.get)

        services.manage()

        self.assertTrue(self.render_template.called)
        self.assertTrue(self.apply.called)
        self.assertTrue(service_restart.called)

    def _full_rids(self, name=None):
        """Returns a random id for each of the 4 relations it establishes."""
        if name is None:
            return self.data.keys()
        rids = []
        for key in self.data.keys():
            if key.startswith(name) and key[len(name)] == ':':
                rids.append(key)
        return rids

    def _related_units(self, rid=None):
        if rid is None:
            return [unit for unit in self.data.values()]
        return self.data[rid].keys()

    def _full_relation_data(self):
        return {
            'midonet:6': {
                'midonet-api/0': {
                    'host': 'mido_addr',
                    'port': '8080',
                },
                'midonet-api/1': {
                    'host': 'mido_addr',
                    'port': '8080',
                },
            },
        }

    def _relation_get_full(self, rid=None, unit=None):
        return self.data[rid][unit]
